@@ -284,6 +284,9 @@ circos.heatmap.validate = function(mat_list) {
 # -col If the values in the matrices are continuous, the color should be a color mapping generated by 
 #      `colorRamp2`. If the values are characters, the color should be a named color vector.
 # -na.col Color for ``NA`` values.
+# -cell.border Border color of cells. A single scalar.
+# -cell.lty Line type of cell borders. A single scalar.
+# -cell.lwd Line width of cell borders. A single scalar.
 # -bg.border Color for background border.
 # -bg.lty Line type of the background border.
 # -bg.lwd Line width of the background border.
@@ -324,7 +327,9 @@ circos.heatmap.validate = function(mat_list) {
 # circos.clear()
 # }
 circos.heatmap = function(mat, split = NULL, col, na.col = "grey", 
-	bg.border = NA, bg.lty = par("lty"), bg.lwd = par("lwd"), ignore.white = TRUE, 
+	cell.border = NA, cell.lty = 1, cell.lwd = 1,
+	bg.border = NA, bg.lty = par("lty"), bg.lwd = par("lwd"), 
+	ignore.white = is.na(cell.border), 
 	cluster = TRUE, clustering.method = "complete", distance.method = "euclidean",
 	dend.callback = function(dend, m, si) reorder(dend, rowMeans(m)),
 	dend.side = c("none", "outside", "inside"), dend.track.height = 0.1,
@@ -436,7 +441,15 @@ circos.heatmap = function(mat, split = NULL, col, na.col = "grey",
 	}
 	
 	if(is.function(col)) {
-		col_fun = col
+		col_fun = function(x) {
+			attr = attributes(x)
+			l = is.na(x)
+			v = character(length(x))
+			v[!l] = col(x[!l])
+			v[l] = na.col
+			attributes(v) = attr
+			v 
+		}
 	} else {
 		if(is.null(names(col))) {
 			stop_wrap('`col` should be a named vector.')
@@ -468,28 +481,45 @@ circos.heatmap = function(mat, split = NULL, col, na.col = "grey",
 	    nr = nrow(m2)
 	    nc = ncol(m2)
 
+	    if(ignore.white) {
+		    if(any(grepl("#FFFFFF", col_mat)) && !is.na(cell.border)) {
+				stop_wrap("`ignore.white` must be set to `FALSE` when `cell.border` is set.")
+			}
+		}
+
 	    for(i in 1:nc) {
 	    	if(ignore.white) {
 	    		l = grepl("#FFFFFF", col_mat[, i])  # white colors
+
 	    		if(all(l)) {
 	    			# no nothing
 	    		} else {
 	    			# qqcat("@{sum(l)} white rectangles are not drawn.\n")
+	    			if(is.na(cell.border)) {
+		    			cell.border2 = col_mat[, i][!l]
+		    		} else {
+		    			cell.border2 = cell.border
+		    		}
 	    			circos.rect(
 	    				(CELL_META$cell_middle - CELL_META$cell_width/2)[od][!l], 
 	    				(rep(nc - i, nr))[!l], 
 			            (CELL_META$cell_middle + CELL_META$cell_width/2)[od][!l], 
 			            (rep(nc - i + 1, nr))[!l], 
-			            border = col_mat[, i][!l], 
+			            border = cell.border2, lty = cell.lty, lwd = cell.lwd,
 			            col = col_mat[, i][!l])
 	    		}
 	    	} else {
+	    		if(is.na(cell.border)) {
+	    			cell.border2 = col_mat[, i]
+	    		} else {
+	    			cell.border2 = cell.border
+	    		}
 		        circos.rect(
 		        	(CELL_META$cell_middle - CELL_META$cell_width/2)[od], 
 		        	rep(nc - i, nr), 
 		            (CELL_META$cell_middle + CELL_META$cell_width/2)[od], 
 		            rep(nc - i + 1, nr), 
-		            border = col_mat[, i], col = col_mat[, i])
+		            border = cell.border2, col = col_mat[, i], lty = cell.lty, lwd = cell.lwd)
 		       }
 	    }
 
