@@ -75,16 +75,32 @@ circos.link = function(
     arr.col = col,
     reduce_to_mid_line = FALSE) {
 
-    sector.data1 = get.sector.data(sector.index1)
-    sector.data2 = get.sector.data(sector.index2)
+   sector.data1 = get.sector.data(sector.index1)
+   sector.data2 = get.sector.data(sector.index2)
+
+   if(circos.par$ring) {
+   	if(length(point1) == 2) {
+   		if(point1[1] > point1[2]) {
+   			point1[1] = point1[1] - diff(get.sector.data()[c("min.value", "max.value")])
+   		}
+   	}
+
+   	if(length(point2) == 2) {
+   		if(point2[1] > point2[2]) {
+   			point2[1] = point2[1] - diff(get.sector.data()[c("min.value", "max.value")])
+   		}
+   	}
+   }
 
 	point1 = sort(point1)
 	point2 = sort(point2)
 
-	if(min(point1) < sector.data1["min.data"] || max(point1) > sector.data1["max.data"]) {
+	rg1 = sector.data1["max.data"] - sector.data1["min.data"]
+	if(min(point1) < sector.data1["min.data"] - rg1*1e-6 || max(point1) > sector.data1["max.data"] + rg1*1e-6) {
 		message_wrap(paste0("Note: The first link end is drawn out of sector '", sector.index1, "'."))
 	}
-	if(min(point2) < sector.data2["min.data"] || max(point2) > sector.data2["max.data"]) {
+	rg2 = sector.data2["max.data"] - sector.data2["min.data"]
+	if(min(point2) < sector.data2["min.data"] - rg2*1e-6 || max(point2) > sector.data2["max.data"] + rg2*1e-6) {
 		message_wrap(paste0("Note: The second link end is drawn out of sector '", sector.index2, "'."))
 	}
 
