@@ -4,8 +4,8 @@
 # == param
 # -x            Data points on x-axis. The value can also be a two-column matrix/data frame if you put x and y data points into one variable.
 # -y            Data points on y-axis.
-# -sector.index Index for the sector to convert the coordinates
-# -track.index  Index for the track to convert the coordinates
+# -sector.index Index for the sector to convert the coordinates.
+# -track.index  Index for the track to convert the coordinates.
 #
 # == details
 # This is the core function in the package. It transform data points from data coordinate system (in a specific cell) to the polar coordinate system.
@@ -13,8 +13,19 @@
 # == values
 # A matrix with two columns (``theta`` and ``rou``). ``rou`` is measured in degree.
 #
-circlize = function(x, y, sector.index = get.current.sector.index(),
-	track.index = get.current.track.index()) {
+# == example
+# pdf(NULL)
+# sectors = c("a", "b")
+# circos.initialize(sectors, xlim = c(0, 1))
+# circos.track(ylim = c(0, 1))
+# # x = 0.5, y = 0.5 in sector a and track 1
+# circlize(0.5, 0.5, sector.index = "a", track.index = 1)
+# circos.clear()
+# dev.off()
+circlize = function(
+    x, y, 
+    sector.index = get.current.sector.index(),
+    track.index = get.current.track.index()) {
 
     if(missing(y)) {
         if(ncol(x) >= 2) {
@@ -27,6 +38,10 @@ circlize = function(x, y, sector.index = get.current.sector.index(),
        
     theta = sector.data["start.degree"] - (x - sector.data["min.value"]) / (sector.data["max.value"] - sector.data["min.value"]) *
             abs(sector.data["start.degree"] - sector.data["end.degree"])
+
+    if(!circos.par("xaxis.clock.wise")) {
+        theta = sector.data["start.degree"] - theta + sector.data["end.degree"]
+    }
     
 	if(track.index == 0) {
 		rou = rep(1, length(theta))
@@ -58,7 +73,20 @@ circlize = function(x, y, sector.index = get.current.sector.index(),
 #
 # == values
 # A matrix with two columns (``x`` and ``y``)
-reverse.circlize = function(x, y, sector.index = get.current.sector.index(),
+# 
+# == example
+# pdf(NULL)
+# sectors = letters[1:4]
+# circos.initialize(sectors, xlim = c(0, 1))
+# circos.trackPlotRegion(ylim = c(0, 1))
+# reverse.circlize(c(30, 60), c(0.9, 0.8))
+# reverse.circlize(c(30, 60), c(0.9, 0.8), sector.index = "d", track.index = 1)
+# reverse.circlize(c(30, 60), c(0.9, 0.8), sector.index = "a", track.index = 1)
+# circos.clear()
+# dev.off()
+reverse.circlize = function(
+    x, y, 
+    sector.index = get.current.sector.index(),
     track.index = get.current.track.index()) {
 
     if(missing(y)) {
@@ -386,11 +414,11 @@ circos.approx = function(x, y, resolution = 0.1, sector.index = get.cell.meta.da
 # Add transparency to colors
 #
 # == param
-# -col a vector of colors
-# -transparency transparency, numeric value between 0 and 1
+# -col A vector of colors.
+# -transparency Transparency, numeric value between 0 and 1.
 #
 # == value
-# A vector of colors
+# A vector of colors.
 #
 # == example
 # add_transparency("red", 0.5)
@@ -400,6 +428,9 @@ add_transparency = function (col, transparency = 0) {
     rgb(t(col2rgb(col)/255), alpha = 1 - transparency)
 }
 
+# == title
+# Get the inside radius of the most inner track
+#
 get_most_inside_radius = function() {
 	tracks = get.all.track.index()
 	if(length(tracks) == 0) {
@@ -411,15 +442,19 @@ get_most_inside_radius = function() {
 }
 
 # == title
-# Convert adjacency list to adjacency matrix
+# Convert adjacency list to an adjacency matrix
 #
 # == param
-# -lt a data frame which contains adjacency list.
-# -square is the returned matrix a square matrix?
-#
-# == details
-# Convert adjacency list to adjacency matrix.
+# -lt A data frame which contains adjacency list.
+# -square Should the returned matrix be a square matrix?
 #
+# == example
+# set.seed(123)
+# df = data.frame(from = sample(letters, 10, replace = TRUE), 
+#                 to = sample(letters, 10, replace = TRUE), 
+#                 value = 1:10)
+# adjacencyList2Matrix(df)
+# adjacencyList2Matrix(df, square = TRUE)
 adjacencyList2Matrix = function(lt, square = FALSE) {
 	lt = as.data.frame(lt)
 	if(ncol(lt) == 2) {
@@ -458,6 +493,30 @@ adjacencyList2Matrix = function(lt, square = FALSE) {
 }
 
 
+# == title
+# Convert adjacency matrix to an adjacency list
+#
+# == param
+# -mat A numeric matrix.
+# -keep.zero Whether to keep the interactions with value zero.
+#
+# == example
+# set.seed(999)
+# mat = matrix(sample(18, 18), 3, 6) 
+# rownames(mat) = paste0("S", 1:3)
+# colnames(mat) = paste0("E", 1:6)
+# adjacencyMatrix2List(mat)
+adjacencyMatrix2List = function(mat, keep.zero = FALSE) {
+    if(is.null(rownames(mat))) rownames(mat) = as.character(seq_len(nrow(mat)))
+    if(is.null(colnames(mat))) colnames(mat) = as.character(seq_len(ncol(mat)))
+    df = data.frame(from = rep(rownames(mat), times = ncol(mat)),
+        to = rep(colnames(mat), each = nrow(mat)),
+        value = as.vector(mat),
+        stringsAsFactors = FALSE)
+    if(!keep.zero) df = df[df$value != 0, , drop = FALSE]
+    return(df)
+}
+
 
 # == title
 # Transform back from colors to values
@@ -481,7 +540,7 @@ adjacencyList2Matrix = function(lt, square = FALSE) {
 # Zuguang Gu <z.gu@dkfz.de>
 #
 # == example
-# x = seq(0, 1, length = 11)
+# x = seq(0, 1, length.out = 11)
 # col_fun = colorRamp2(c(0, 0.5, 1), c("blue", "white", "red"))
 # col = col_fun(x)
 # col2value(col, col_fun = col_fun)
@@ -551,21 +610,25 @@ col2value = function(r, g, b, col_fun) {
 # Since in the circular plot, the aspect ratio is always 1, it does not matter this conversion
 # is applied on x direction or y direction.
 #
+# This function is mainly used in the radical direction.
+#
 # == seealso
 # `convert_x` and `convert_y` convert absolute units into a data coordinate in a specified cell.
 #
+# https://jokergoo.github.io/circlize_book/book/circular-layout.html#convert-functions
+#
 # == author
 # Zuguang Gu <z.gu@dkfz.de>
 #
 # == example
-# fa = letters[1:10]
+# sectors = letters[1:10]
 # circos.par(cell.padding = c(0, 0, 0, 0), track.margin = c(0, 0))
-# circos.initialize(fa, xlim = cbind(rep(0, 10), runif(10, 0.5, 1.5)))
-# circos.track(ylim = c(0, 1), track.height = convert_length(5, "mm"))
-# circos.par(track.margin = c(0, convert_length(2, "mm")))
-# circos.track(ylim = c(0, 1), track.height = convert_length(1, "cm"))
-# circos.par(track.margin = c(0, convert_length(5, "mm")))
-# circos.track(ylim = c(0, 1), track.height = convert_length(1, "inches"))
+# circos.initialize(sectors, xlim = cbind(rep(0, 10), runif(10, 0.5, 1.5)))
+# circos.track(ylim = c(0, 1), track.height = mm_h(5))
+# circos.par(track.margin = c(0, mm_h(2)))
+# circos.track(ylim = c(0, 1), track.height = cm_h(1))
+# circos.par(track.margin = c(0, mm_h(5)))
+# circos.track(ylim = c(0, 1), track.height = inch_h(1))
 # circos.clear()
 convert_length = function(x, unit = c("mm", "cm", "inches")) {
 
@@ -604,6 +667,9 @@ convert_length = function(x, unit = c("mm", "cm", "inches")) {
 # This function is same as `convert_length`. The reason for naming this function
 # is `convert_length` is mostely used for defining the height of tracks and track margins.
 #
+# == seealso
+# For pre-defined units, users can use `cm_h`, `mm_h` and `inches_h`.
+#
 # == author
 # Zuguang Gu <z.gu@dkfz.de>
 #
@@ -618,17 +684,11 @@ convert_height = function(...) {
 # Convert units
 #
 # == param
-# -... pass to `convert_length`
+# -... pass to `convert_length`.
 #
 # == details
-# This function is same as `convert_length`.
+# Please do not use this function. Use `mm_h`/`cm_h`/inches_h` instead.
 #
-# == author
-# Zuguang Gu <z.gu@dkfz.de>
-#
-# == example
-# # see example in `convert_length` page
-# NULL
 uh = function(...) {
     convert_length(...)
 }
@@ -694,45 +754,51 @@ convert_unit_in_data_coordinate = function(x, unit = c("mm", "cm", "inches", "ca
 # Convert unit on x direction in data coordinate
 #
 # == param
-# -x a numeric vector
-# -unit supported units, only "mm", "cm", "inches"
-# -sector.index index for the sector where the conversion is applied
-# -track.index index for the track where the conversion is applied
+# -x a numeric vector.
+# -unit supported units, only "mm", "cm", "inches".
+# -sector.index index for the sector where the conversion is applied.
+# -track.index index for the track where the conversion is applied.
 # -h since the width of the cell is not identical from the top to the bottom in the cell, the position on
-#   y direction needs to be specified. By default it is at the middle point on y-axis
+#   y direction needs to be specified. By default it is at the middle point on y-axis.
 #
 # == value
-# A vector of numeric values which are measured in the specified data coordinate
+# A vector of numeric values which are measured in the specified data coordinate.
 #
 # == seealso
+# For pre-defined units, users can use `cm_x`, `mm_x` and `inches_x`.
+#
 # `convert_y` converts on y direction.
 #
+# https://jokergoo.github.io/circlize_book/book/circular-layout.html#convert-functions
+#
 # == author
 # Zuguang Gu <z.gu@dkfz.de>
 #
 # == example
-# fa = letters[1:10]
+# sectors = letters[1:10]
 # circos.par(cell.padding = c(0, 0, 0, 0), track.margin = c(0, 0))
-# circos.initialize(fa, xlim = cbind(rep(0, 10), runif(10, 0.5, 1.5)))
-# circos.track(ylim = c(0, 1), track.height = convert_height(5, "mm"),
+# circos.initialize(sectors, xlim = cbind(rep(0, 10), runif(10, 0.5, 1.5)))
+# circos.track(ylim = c(0, 1), track.height = mm_h(5),
 #     panel.fun = function(x, y) {
-#         circos.lines(c(0, 0 + convert_x(5, "mm")), c(0.5, 0.5), col = "blue")
+#         circos.lines(c(0, 0 + mm_x(5)), c(0.5, 0.5), col = "blue")
 #     })
-# circos.par(track.margin = c(0, convert_height(2, "mm")))
-# circos.track(ylim = c(0, 1), track.height = convert_height(1, "cm"),
+# circos.par(track.margin = c(0, mm_h(2)))
+# circos.track(ylim = c(0, 1), track.height = cm_h(1),
 #     panel.fun = function(x, y) {
 #         xcenter = get.cell.meta.data("xcenter")
-#         circos.lines(c(xcenter, xcenter), c(0, convert_y(1, "cm")), col = "red")
+#         circos.lines(c(xcenter, xcenter), c(0, cm_y(1)), col = "red")
 #     })
-# circos.par(track.margin = c(0, convert_height(5, "mm")))
-# circos.track(ylim = c(0, 1), track.height = convert_height(1, "inches"),
+# circos.par(track.margin = c(0, mm_h(5)))
+# circos.track(ylim = c(0, 1), track.height = inch_h(1),
 #     panel.fun = function(x, y) {
-#         line_length_on_x = convert_x(1*sqrt(2)/2, "cm")
-#         line_length_on_y = convert_y(1*sqrt(2)/2, "cm")
+#         line_length_on_x = cm_x(1*sqrt(2)/2)
+#         line_length_on_y = cm_y(1*sqrt(2)/2)
 #         circos.lines(c(0, line_length_on_x), c(0, line_length_on_y), col = "orange")
 #     })
 # circos.clear()
-convert_x = function(x, unit = c("mm", "cm", "inches"),
+convert_x = function(
+    x, 
+    unit = c("mm", "cm", "inches"),
     sector.index = get.cell.meta.data("sector.index"),
     track.index = get.cell.meta.data("track.index"),
     h = get.cell.meta.data("ycenter", sector.index = sector.index, 
@@ -745,17 +811,11 @@ convert_x = function(x, unit = c("mm", "cm", "inches"),
 # Convert unit on x direction in data coordinate
 #
 # == param
-# -... pass to `convert_x`
+# -... pass to `convert_x`.
 #
 # == details
-# This function is same as `convert_x`.
+# Please do not use this function. Use `mm_x`/`cm_x`/inches_x` instead.
 #
-# == author
-# Zuguang Gu <z.gu@dkfz.de>
-#
-# == example
-# # see example in `convert_x` page
-# NULL
 ux = function(...) {
     convert_x(...)
 }
@@ -773,15 +833,21 @@ ux = function(...) {
 # A vector of numeric values which are measured in the specified data coordinate
 #
 # == seealso
+# For pre-defined units, users can use `cm_y`, `mm_y` and `inches_y`.
+#
 # `convert_x` converts on x direction.
 #
+# https://jokergoo.github.io/circlize_book/book/circular-layout.html#convert-functions
+#
 # == author
 # Zuguang Gu <z.gu@dkfz.de>
 #
 # == example
 # # see example on `convert_x` page 
 # NULL
-convert_y = function(x, unit = c("mm", "cm", "inches"),
+convert_y = function(
+    x, 
+    unit = c("mm", "cm", "inches"),
     sector.index = get.current.sector.index(),
     track.index = get.current.track.index()) {
     convert_unit_in_data_coordinate(x, unit = unit, sector.index = sector.index, 
@@ -792,17 +858,11 @@ convert_y = function(x, unit = c("mm", "cm", "inches"),
 # Convert unit on y direction in data coordinate
 #
 # == param
-# -... pass to `convert_y`
+# -... pass to `convert_y`.
 #
 # == details
-# This function is same as `convert_y`.
+# Please do not use this function. Use `mm_y`/`cm_y`/inches_y` instead.
 #
-# == author
-# Zuguang Gu <z.gu@dkfz.de>
-#
-# == example
-# # see example in `convert_y` page
-# NULL
 uy = function(...) {
     convert_y(...)
 }
@@ -833,6 +893,233 @@ convert_unit_in_canvas_coordinate = function(x, unit = c("mm", "cm", "inches"))
     return(len)
 }
 
+# == title
+# Convert unit on x direction in data coordinate
+#
+# == param
+# -x The x-value in numeric.
+# -sector.index Index of sector.
+# -track.index Index of track.
+# -... Pass to `convert_x`.
+#
+# == details
+# See explanations in `convert_x` page.
+#
+# == author
+# Zuguang Gu <z.gu@dkfz.de>
+#
+# == example
+# # see examples in `convert_x` page
+# NULL
+cm_x = function(x, sector.index = get.current.sector.index(),
+    track.index = get.current.track.index(), ...) {
+    convert_x(x, unit = "cm", sector.index = sector.index, track.index = track.index, ...)
+}
+
+# == title
+# Convert unit on y direction in data coordinate
+#
+# == param
+# -y The y-value in numeric.
+# -sector.index Index of sector.
+# -track.index Index of track.
+#
+# == details
+# See explanations in `convert_y` page.
+#
+# == author
+# Zuguang Gu <z.gu@dkfz.de>
+#
+# == example
+# # see examples in `convert_y` page
+# NULL
+cm_y = function(y, sector.index = get.current.sector.index(),
+    track.index = get.current.track.index()) {
+    convert_y(y, unit = "cm", sector.index = sector.index, track.index = track.index)
+}
+
+# == title
+# Convert units
+#
+# == param
+# -h The height in numeric.
+#
+# == details
+# See explanations in `convert_length` page.
+#
+# == author
+# Zuguang Gu <z.gu@dkfz.de>
+#
+# == example
+# # see examples in `convert_length` page
+# NULL
+cm_h = function(h) {
+    convert_length(h, unit = "cm")
+}
+
+# == title
+# Convert unit on x direction in data coordinate
+#
+# == param
+# -x The x-value in numeric.
+# -sector.index Index of sector.
+# -track.index Index of track.
+# -... Pass to `convert_x`.
+#
+# == details
+# See explanations in `convert_x` page.
+#
+# == author
+# Zuguang Gu <z.gu@dkfz.de>
+#
+# == example
+# # see examples in `convert_x` page
+# NULL
+mm_x = function(x, sector.index = get.current.sector.index(),
+    track.index = get.current.track.index(), ...) {
+    convert_x(x, unit = "mm", sector.index = sector.index, track.index = track.index, ...)
+}
+
+# == title
+# Convert unit on y direction in data coordinate
+#
+# == param
+# -y The y-value in numeric.
+# -sector.index Index of sector.
+# -track.index Index of track.
+#
+# == details
+# See explanations in `convert_y` page.
+#
+# == author
+# Zuguang Gu <z.gu@dkfz.de>
+#
+# == example
+# # see examples in `convert_y` page
+# NULL
+mm_y = function(y, sector.index = get.current.sector.index(),
+    track.index = get.current.track.index()) {
+    convert_y(y, unit = "mm", sector.index = sector.index, track.index = track.index)
+}
+
+# == title
+# Convert units
+#
+# == param
+# -h The height in numeric.
+#
+# == details
+# See explanations in `convert_length` page.
+#
+# == author
+# Zuguang Gu <z.gu@dkfz.de>
+#
+# == example
+# # see examples in `convert_length` page
+# NULL
+mm_h = function(h) {
+    convert_length(h, unit = "mm")
+}
+
+# == title
+# Convert unit on x direction in data coordinate
+#
+# == param
+# -x The x-value in numeric.
+# -sector.index Index of sector.
+# -track.index Index of track.
+# -... Pass to `convert_x`.
+#
+# == details
+# See explanations in `convert_x` page.
+#
+# == author
+# Zuguang Gu <z.gu@dkfz.de>
+#
+# == example
+# # see examples in `convert_x` page
+# NULL
+inches_x = function(x, sector.index = get.current.sector.index(),
+    track.index = get.current.track.index(), ...) {
+    convert_x(x, unit = "inches", sector.index = sector.index, track.index = track.index, ...)
+}
+
+# == title
+# Convert unit on y direction in data coordinate
+#
+# == param
+# -y The y-value in numeric.
+# -sector.index Index of sector.
+# -track.index Index of track.
+#
+# == details
+# See explanations in `convert_y` page.
+#
+# == author
+# Zuguang Gu <z.gu@dkfz.de>
+#
+# == example
+# # see examples in `convert_y` page
+# NULL
+inches_y = function(y, sector.index = get.current.sector.index(),
+    track.index = get.current.track.index()) {
+    convert_y(y, unit = "inches", sector.index = sector.index, track.index = track.index)
+}
+
+# == title
+# Convert units
+#
+# == param
+# -h The height in numeric.
+#
+# == details
+# See explanations in `convert_length` page.
+#
+# == author
+# Zuguang Gu <z.gu@dkfz.de>
+#
+# == example
+# # see examples in `convert_length` page
+# NULL
+inches_h = function(h) {
+    convert_length(h, unit = "inches")
+}
+
+# == title
+# Convert unit on x direction in data coordinate
+#
+# == param
+# -... pass to `inches_x`.
+#
+# == details
+# This function is the same as `inches_x`.
+inch_x = function(...) {
+    inches_x(...)
+}
+# == title
+# Convert unit on y direction in data coordinate
+#
+# == param
+# -... pass to `inches_y`
+#
+# == details
+# This function is the same as `inches_y`.
+inch_y = function(...) {
+    inches_y(...)
+}
+
+# == title
+# Convert units
+#
+# == param
+# -... pass to `inches_h`
+#
+# == details
+# This function is the same as `inches_h`.
+inch_h = function(...) {
+    inches_h(...)
+}
+
 stop_wrap = function(...) {
     x = paste0(...)
     x = paste(strwrap(x), collapse = "\n")
@@ -853,7 +1140,7 @@ message_wrap = function(...) {
 
 validate_data_frame = function(x) {
     if(inherits(x, "data.frame")) {
-        return(x)
+        return(as.data.frame(x))
     } else if(inherits(x, "GRanges")) {
         x = as.data.frame(x)
         return(x[, -(4:5), drop = FALSE])
@@ -865,3 +1152,89 @@ validate_data_frame = function(x) {
         return(x)
     }
 }
+
+
+roundrect_pos = function(xleft, ybottom, xright, ytop, radius = 0.1, 
+  asp = NULL) {
+
+  if(is.null(asp)) {
+    pin = par("pin")
+    usr = par("usr")
+    asp = pin[1]/pin[2] * (usr[4] - usr[3])/(usr[2] - usr[1])
+  }
+
+  convert_y_1 = function(y, asp) {
+    y/asp
+  }
+
+  covnert_y_2 = function(y, asp) {
+    y*asp
+  }
+
+  if(xleft > xright) {
+    foo = xleft
+    xleft = xright
+    xright = foo
+  }
+  if(ybottom > ytop) {
+    foo = ybottom
+    ybottom = ytop
+    ytop = foo
+  }
+
+  ytop = convert_y_1(ytop, asp)
+  ybottom = convert_y_1(ybottom, asp)
+
+  w = abs(xright - xleft)
+  h = abs(ytop - ybottom)
+  r = radius*min(w, h)
+
+  x = NULL
+  y = NULL
+
+  # left top 180 -> 90
+  df = polar2Cartesian(cbind(seq(180, 90, length.out = 20), rep(r,20)))
+  x = c(x, df[, 1] + xleft + r)
+  y = c(y, df[, 2] + ytop - r)
+
+  x = c(x, xleft + r, xright - r)
+  y = c(y, ytop, ytop)
+
+  # right top 90 -> 0
+  df = polar2Cartesian(cbind(seq(90, 0, length.out = 20), rep(r,20)))
+  x = c(x, df[, 1] + xright - r)
+  y = c(y, df[, 2] + ytop - r)
+
+  x = c(x, xright, xright)
+  y = c(y, ytop - r, ybottom + r)
+
+  # bottom right 0 - (-90)
+  df = polar2Cartesian(cbind(seq(0, -90, length.out = 20), rep(r,20)))
+  x = c(x, df[, 1] + xright - r)
+  y = c(y, df[, 2] + ybottom + r)
+
+  x = c(x, xright - r, xleft + r)
+  y = c(y, ybottom, ybottom)
+
+  # bottom left 270 - 180
+  df = polar2Cartesian(cbind(seq(270, 180, length.out = 20), rep(r,20)))
+  x = c(x, df[, 1] + xleft + r)
+  y = c(y, df[, 2] + ybottom + r)
+
+  x = c(x, xleft, xleft)
+  y = c(y, ybottom + r, ytop - r)
+
+  y = covnert_y_2(y, asp)
+
+  return(list(x = x, y = y))
+}
+
+# plot(c(-2, -1), c(-2, -1))
+# foo = roundrect_pos(-2, -2, -1, -1)
+# lines(foo$x, foo$y)
+
+
+# plot(c(-4, -1), c(-2, -1))
+# foo = roundrect_pos(-4, -2, -1, -1)
+# lines(foo$x, foo$y)
+
